@@ -18,7 +18,7 @@
 */
 
 /*
- * $Id: ccid.h,v 1.2 2005-11-27 20:12:17 rousseau Exp $
+ * $Id: ccid.h,v 1.5 2006/04/22 16:04:28 rousseau Exp $
  */
 
 typedef struct
@@ -86,6 +86,10 @@ typedef struct
 	 */
 	unsigned int readTimeout;
 
+	/*
+	 * Card protocol
+	 */
+	int cardProtocol;
 } _ccid_descriptor;
 
 /* Features from dwFeatures */
@@ -118,19 +122,41 @@ typedef struct
 #define GEMPC433	0x08E64433
 #define GEMPCKEY	0x08E63438
 #define GEMPCTWIN	0x08E63437
+#define GEMPCPINPAD 0x08E63478
+#define GEMCORESIMPRO 0x08E63480
+#define GEMCOREPOSPRO 0x08E63479
 #define CARDMAN3121	0x076B3021
 #define LTC31		0x07830003
 #define SCR331DI	0x04E65111
 #define SCR331DINTTCOM	0x04E65120
+#define SDI010		0x04E65121
 #define CHERRYXX33	0x046A0005
 #define CHERRYST2000	0x046A003E
 #define OZ776		0x0B977762
 #define SPR532		0x04E6E003
+#define MYSMARTPAD	0x09BE0002
+#define CHERRYXX44	0x046a0010
 
 /* Escape sequence codes */
 #define ESC_GEMPC_SET_ISO_MODE		1
 #define ESC_GEMPC_SET_APDU_MODE		2
 
+/*
+ * Possible values :
+ * 3 -> 1.8V, 3V, 5V
+ * 2 -> 3V, 5V
+ * 1 -> 5V only
+ * 0 -> automatic (selection made by the reader)
+ */
+/*
+ * To be safe we default to 5V
+ * otherwise we would have to parse the ATR and get the value of TAi (i>2) when
+ * in T=15
+ */
+#define VOLTAGE_AUTO 0
+#define VOLTAGE_5V 1
+#define VOLTAGE_3V 2
+#define VOLTAGE_1_8V 3
 
 int ccid_open_hack(unsigned int reader_index);
 void ccid_error(int error, char *file, int line, const char *function);
@@ -139,3 +165,17 @@ _ccid_descriptor *get_ccid_descriptor(unsigned int reader_index);
 /* convert a 4 byte integer in USB format into an int */
 #define dw2i(a, x) ((((((a[x+3] << 8) + a[x+2]) << 8) + a[x+1]) << 8) + a[x])
 
+/* all the data rates specified by ISO 7816-3 Fi/Di tables */
+#define ISO_DATA_RATES 10753, 14337, 15625, 17204, \
+		20833, 21505, 23438, 25806, 28674, \
+		31250, 32258, 34409, 39063, 41667, \
+		43011, 46875, 52083, 53763, 57348, \
+		62500, 64516, 68817, 71685, 78125, \
+		83333, 86022, 93750, 104167, 107527, \
+		114695, 125000, 129032, 143369, 156250, \
+		166667, 172043, 215054, 229391, 250000, \
+		344086
+
+/* data rates supported by the secondary slots on the GemCore Pos Pro & SIM Pro */
+#define GEMPLUS_CUSTOM_DATA_RATES 10753, 21505, 43011, 125000
+
