@@ -18,7 +18,7 @@
 */
 
 /*
- * $Id: ccid_usb.c 1703 2005-11-07 15:46:03Z rousseau $
+ * $Id: ccid_usb.c 2014 2006-04-18 15:05:33Z rousseau $
  */
 
 #define __CCID_USB__
@@ -31,6 +31,7 @@
 # endif
 #include <usb.h>
 
+#include "misc.h"
 #include "ccid.h"
 #include "config.h"
 #include "debug.h"
@@ -73,6 +74,10 @@ typedef struct
 	int bulk_in;
 	int bulk_out;
 
+	/* Number of slots using the same device */
+	int real_nb_opened_slots;
+	int *nb_opened_slots;
+
 	/*
 	 * CCID infos common to USB and serial
 	 */
@@ -109,8 +114,14 @@ static struct _bogus_firmware Bogus_firmwares[] = {
 	{ 0x04e6, 0xe003, 0x0504 },	/* SPR 532 */
 	{ 0x0D46, 0x3001, 0x0037 },	/* KAAN Base */
 	{ 0x0D46, 0x3002, 0x0037 },	/* KAAN Advanced */
+	{ 0x09C3, 0x0008, 0x0203 },	/* ActivCard V2 */
+	{ 0x0DC3, 0x1004, 0x0502 },	/* ASE IIIe USBv2 */
+	{ 0x0DC3, 0x1102, 0x0607 },	/* ASE IIIe KB USB */
 };
 
+/* data rates supported by the secondary slots on the GemCore Pos Pro & SIM Pro */
+unsigned int SerialCustomDataRates[] = { GEMPLUS_CUSTOM_DATA_RATES, 0 };
+
 
 /*****************************************************************************
  *
@@ -217,23 +228,23 @@ status_t OpenUSBByName(unsigned int reader_index, /*@null@*/ char *device)
 	/* general driver info */
 	if (!LTPBundleFindValueWithKey(infofile, "ifdManufacturerString", keyValue, 0))
 	{
-		DEBUG_CRITICAL2("Manufacturer: %s", keyValue);
+		DEBUG_INFO2("Manufacturer: %s", keyValue);
 	}
 	else
 	{
-		DEBUG_CRITICAL2("LTPBundleFindValueWithKey error. Can't find %s?",
+		DEBUG_INFO2("LTPBundleFindValueWithKey error. Can't find %s?",
 			infofile);
 		return STATUS_UNSUCCESSFUL;
 	}
 	if (!LTPBundleFindValueWithKey(infofile, "ifdProductString", keyValue, 0))
 	{
-		DEBUG_CRITICAL2("ProductString: %s", keyValue);
+		DEBUG_INFO2("ProductString: %s", keyValue);
 	}
 	else
 		return STATUS_UNSUCCESSFUL;
 	if (!LTPBundleFindValueWithKey(infofile, "Copyright", keyValue, 0))
 	{
-		DEBUG_CRITICAL2("Copyright: %s", keyValue);
+		DEBUG_INFO2("Copyright: %s", keyValue);
 	}
 	else
 		return STATUS_UNSUCCESSFUL;
@@ -301,6 +312,7 @@ status_t OpenUSBByName(unsigned int reader_index, /*@null@*/ char *device)
 					if (already_used)
 					{
 						if ((previous_reader_index != -1)
+							&& usbDevice[previous_reader_index].dev
 							&& (strcmp(usbDevice[previous_reader_index].dev->bus->dirname, bus->dirname)  == 0)
 							&& (strcmp(usbDevice[previous_reader_index].dev->filename, dev->filename) == 0)
 							&& usbDevice[previous_reader_index].ccid.bCurrentSlotIndex < usbDevice[previous_reader_index].ccid.bMaxSlotIndex)
@@ -308,7 +320,15 @@ status_t OpenUSBByName(unsigned int reader_index, /*@null@*/ char *device)
 							/* we reuse the same device
 							 * and the reader is multi-slot */
 							usbDevice[reader_index] = usbDevice[previous_reader_index];
-							usbDevice[reader_index].ccid.pbSeq = usbDevice[previous_reader_index].ccid.pbSeq;
+							/* the other slots do not have the same data rates */
+							if ((GEMCOREPOSPRO == usbDevice[reader_index].ccid.readerID)
+								|| (GEMCORESIMPRO == usbDevice[reader_index].ccid.readerID))
+							{
+								usbDevice[reader_index].ccid.arrayOfSupportedDataRates = SerialCustomDataRates;
+								usbDevice[reader_index].ccid.dwMaxDataRate = 125000;
+							}
+
+							*usbDevice[reader_index].nb_opened_slots += 1;
 							usbDevice[reader_index].ccid.bCurrentSlotIndex++;
 							DEBUG_INFO2("Opening slot: %d",
 								usbDevice[reader_index].ccid.bCurrentSlotIndex);
@@ -390,6 +410,8 @@ status_t OpenUSBByName(unsigned int reader_index, /*@null@*/ char *device)
 					usbDevice[reader_index].handle = dev_handle;
 					usbDevice[reader_index].dev = dev;
 					usbDevice[reader_index].interface = interface;
+					usbDevice[reader_index].real_nb_opened_slots = 1;
+					usbDevice[reader_index].nb_opened_slots = &usbDevice[reader_index].real_nb_opened_slots;
 
 					/* CCID common informations */
 					usbDevice[reader_index].ccid.real_bSeq = 0;
@@ -510,16 +532,29 @@ status_t CloseUSB(unsigned int reader_index)
 		usbDevice[reader_index].dev->bus->dirname,
 		usbDevice[reader_index].dev->filename);
 
-	if (usbDevice[reader_index].ccid.arrayOfSupportedDataRates)
+	if (usbDevice[reader_index].ccid.arrayOfSupportedDataRates
+		&& (usbDevice[reader_index].ccid.bCurrentSlotIndex == 0))
+	{
 		free(usbDevice[reader_index].ccid.arrayOfSupportedDataRates);
+		usbDevice[reader_index].ccid.arrayOfSupportedDataRates = NULL;
+	}
 
-	/* reset so that bSeq starts at 0 again */
-	if (DriverOptions & DRIVER_OPTION_RESET_ON_CLOSE)
-		usb_reset(usbDevice[reader_index].handle);
+	/* one slot closed */
+	(*usbDevice[reader_index].nb_opened_slots)--;
 
-	usb_release_interface(usbDevice[reader_index].handle,
-		usbDevice[reader_index].interface);
-	usb_close(usbDevice[reader_index].handle);
+	/* release the allocated ressources for the last slot only */
+	if (0 == *usbDevice[reader_index].nb_opened_slots)
+	{
+		DEBUG_COMM("Last slot closed. Release resources");
+
+		/* reset so that bSeq starts at 0 again */
+		if (DriverOptions & DRIVER_OPTION_RESET_ON_CLOSE)
+			usb_reset(usbDevice[reader_index].handle);
+
+		usb_release_interface(usbDevice[reader_index].handle,
+			usbDevice[reader_index].interface);
+		usb_close(usbDevice[reader_index].handle);
+	}
 
 	/* mark the resource unused */
 	usbDevice[reader_index].handle = NULL;
@@ -598,31 +633,25 @@ static int get_end_points(struct usb_device *dev, _usbDevice *usb_device)
  *					get_ccid_usb_interface
  *
  ****************************************************************************/
-/*@null@*/ struct usb_interface * get_ccid_usb_interface(struct usb_device *dev)
+/*@null@*/ EXTERNAL struct usb_interface * get_ccid_usb_interface(struct usb_device *dev)
 {
 	struct usb_interface *usb_interface = NULL; 
+	int i;
 
 	/* if multiple interfaces use the first one with CCID class type */
-	if (dev->config->bNumInterfaces > 1)
+	for (i=0; i<dev->config->bNumInterfaces; i++)
 	{
-		int ii;
-		for (ii=0; ii<dev->config->bNumInterfaces; ii++)
-		{
-			/* CCID Class? */
-			if (dev->config->interface[ii].altsetting->bInterfaceClass == 0xb
+		/* CCID Class? */
+		if (dev->config->interface[i].altsetting->bInterfaceClass == 0xb
 #ifdef ALLOW_PROPRIETARY_CLASS
-				|| dev->config->interface[ii].altsetting->bInterfaceClass == 0xff
+			|| dev->config->interface[i].altsetting->bInterfaceClass == 0xff
 #endif
-				)
-			{
-				usb_interface = &dev->config->interface[ii];
-				break;
-			}
+			)
+		{
+			usb_interface = &dev->config->interface[i];
+			break;
 		}
 	}
-	else
-		/* only one interface found */
-		usb_interface = dev->config->interface;
 
 #ifdef O2MICRO_OZ776_PATCH
 	if (usb_interface != NULL
@@ -630,8 +659,6 @@ static int get_end_points(struct usb_device *dev, _usbDevice *usb_device)
 		+ dev->descriptor.idProduct)
 		&& (0 == usb_interface->altsetting->extralen)) /* this is the bug */
 	{
-		int i;
-
 		for (i=0; i<usb_interface->altsetting->bNumEndpoints; i++)
 		{
 			/* find the extra[] array */
