@@ -18,7 +18,7 @@
 */
 
 /*
- * $Id: ccid_usb.h 1399 2005-03-14 18:56:53Z rousseau $
+ * $Id: ccid_usb.h 1927 2006-03-06 09:13:56Z rousseau $
  */
 
 status_t OpenUSB(unsigned int reader_index, int channel);
@@ -33,7 +33,7 @@ status_t ReadUSB(unsigned int reader_index, unsigned int *length,
 
 status_t CloseUSB(unsigned int reader_index);
 
-#ifdef __USB_H__
+#if defined (__USB_H__) || defined (_SYS_USB_LIBUSB_USB_H)
 int get_desc(int lun, usb_dev_handle **handle, struct usb_device **dev);
 
 /*@null@*/ struct usb_interface *get_ccid_usb_interface(struct usb_device *dev);
