@@ -18,13 +18,14 @@
 */
 
 /*
- * $Id: commands.c 1747 2005-11-24 15:49:00Z rousseau $
+ * $Id: commands.c 1986 2006-03-28 06:55:31Z rousseau $
  */
 
 #include <string.h>
 #include <stdlib.h>
-#include <PCSC/pcsclite.h>
-#include <PCSC/ifdhandler.h>
+#include <pcsclite.h>
+#include <ifdhandler.h>
+#include <reader.h>
 
 #include "commands.h"
 #include "openct/proto-t1.h"
@@ -34,20 +35,12 @@
 #include "config.h"
 #include "debug.h"
 
-/*
- * Possible values :
- * 3 -> 1.8V, 3V, 5V
- * 2 -> 3V, 5V
- * 1 -> 5V only
- */
-/*
- * To be safe we only use 5V
- * otherwise we would have to parse the ATR and get the value of TAi (i>2) when
- * in T=15
- */
-#define DEFAULT_VOLTAGE 1 /* start with 5 volts */
+/* All the pinpad readers I used are more or less bogus
+ * I use code to change the user command and make the firmware happy */
+#define BOGUS_PINPAD_FIRMWARE
 
 #define max( a, b )   ( ( ( a ) > ( b ) ) ? ( a ) : ( b ) )
+#define offsetof(TYPE, MEMBER) ((size_t) &((TYPE *)0)->MEMBER)
 
 /* internal functions */
 static RESPONSECODE CmdXfrBlockTPDU_T0(unsigned int reader_index,
@@ -71,7 +64,7 @@ static void i2dw(int value, unsigned char *buffer);
  *
  ****************************************************************************/
 RESPONSECODE CmdPowerOn(unsigned int reader_index, unsigned int * nlength,
-	unsigned char buffer[])
+	unsigned char buffer[], int voltage)
 {
 	unsigned char cmd[10];
 	status_t res;
@@ -79,15 +72,12 @@ RESPONSECODE CmdPowerOn(unsigned int reader_index, unsigned int * nlength,
 	unsigned int atr_len;
 	RESPONSECODE return_value = IFD_SUCCESS;
 	_ccid_descriptor *ccid_descriptor = get_ccid_descriptor(reader_index);
-	char voltage;
 
 	/* store length of buffer[] */
 	length = *nlength;
 
 	if (ccid_descriptor->dwFeatures & CCID_CLASS_AUTO_VOLTAGE)
 		voltage = 0;	/* automatic voltage selection */
-	else
-		voltage = DEFAULT_VOLTAGE;
 
 again:
 	cmd[0] = 0x62; /* IccPowerOn */
@@ -135,6 +125,10 @@ again:
 		/* continue with 3 volts and 5 volts */
 		if (voltage > 1)
 		{
+			char *voltage_code[] = { "auto", "5V", "3V", "1.8V" };
+
+			DEBUG_INFO3("Power up with %s failed. Try with %s.",
+				voltage_code[voltage], voltage_code[voltage-1]);
 			voltage--;
 			goto again;
 		}
@@ -161,7 +155,7 @@ again:
  *
  ****************************************************************************/
 RESPONSECODE SecurePINVerify(unsigned int reader_index,
-	const unsigned char TxBuffer[], unsigned int TxLength,
+	unsigned char TxBuffer[], unsigned int TxLength,
 	unsigned char RxBuffer[], unsigned int *RxLength)
 {
 	unsigned char cmd[11+14+CMD_BUF_SIZE];
@@ -182,7 +176,7 @@ RESPONSECODE SecurePINVerify(unsigned int reader_index,
 	 * The equivalent CCID structure is only 14-bytes long */
 	if (TxLength > 19+CMD_BUF_SIZE) /* command too large? */
 	{
-		DEBUG_INFO3("Command too long: %d > %d", TxLength, 16+CMD_BUF_SIZE);
+		DEBUG_INFO3("Command too long: %d > %d", TxLength, 19+CMD_BUF_SIZE);
 		*RxLength = 0;
 		return IFD_NOT_SUPPORTED;
 	}
@@ -196,11 +190,61 @@ RESPONSECODE SecurePINVerify(unsigned int reader_index,
 
 	if (dw2i(TxBuffer, 15) + 19 != TxLength) /* ulDataLength field coherency */
 	{
-		DEBUG_INFO3("Wrong lengths: %d %d", TxBuffer[15] + 19, TxLength);
+		DEBUG_INFO3("Wrong lengths: %d %d", dw2i(TxBuffer, 15) + 19, TxLength);
 		*RxLength = 0;
 		return IFD_NOT_SUPPORTED;
 	}
 
+	/* make sure bEntryValidationCondition is valid
+	 * The Cherry XX44 reader crashes with a wrong value */
+	if ((0x00 == TxBuffer[7]) || (TxBuffer[7] > 0x07))
+	{
+		DEBUG_INFO2("Correct bEntryValidationCondition (was 0x%02X)",
+			TxBuffer[7]);
+		TxBuffer[7] = 0x02;
+	}
+
+#ifdef BOGUS_PINPAD_FIRMWARE
+	/* bug circumvention for the GemPC Pinpad */
+	if (GEMPCPINPAD == ccid_descriptor->readerID)
+	{
+		/* the firmware reject the cases: 00h No string and FFh default
+		 * CCID message. The only value supported is 01h (display 1 message) */
+		if (0x01 != TxBuffer[8])
+		{
+			DEBUG_INFO2("Correct bNumberMessage for GemPC Pinpad (was %d)",
+				TxBuffer[8]);
+			TxBuffer[8] = 0x01;
+		}
+	}
+#endif
+
+	/* T=1 Protocol Management for a TPDU reader */
+	if ((SCARD_PROTOCOL_T1 == ccid_descriptor->cardProtocol)
+		&& (CCID_CLASS_TPDU == (ccid_descriptor->dwFeatures & CCID_CLASS_EXCHANGE_MASK)))
+	{
+		ct_buf_t sbuf;
+		unsigned char sdata[T1_BUFFER_SIZE];
+
+		/* Initialize send buffer with the APDU */
+		ct_buf_set(&sbuf,
+			(void *)(TxBuffer + offsetof(PIN_VERIFY_STRUCTURE, abData)),
+			TxLength - offsetof(PIN_VERIFY_STRUCTURE, abData));
+
+		/* Create T=1 block */  
+		ret = t1_build(&((get_ccid_slot(reader_index))->t1),
+			sdata, 0, T1_I_BLOCK, &sbuf, NULL); 
+
+		/* Increment the sequence numbers  */
+		get_ccid_slot(reader_index)->t1.ns ^= 1;
+		get_ccid_slot(reader_index)->t1.nr ^= 1;
+
+		/* Copy the generated T=1 block prologue into the teoprologue
+		 * of the CCID command */
+		memcpy(TxBuffer + offsetof(PIN_VERIFY_STRUCTURE, bTeoPrologue),
+			sdata, 3);
+	}
+
 	/* Build a CCID block from a PC/SC V2.1.2 Part 10 block */
 	for (a = 11, b = 0; b < TxLength; b++)
 	{
@@ -250,6 +294,27 @@ RESPONSECODE SecurePINVerify(unsigned int reader_index,
 
 	ret = CCID_Receive(reader_index, RxLength, RxBuffer);
 
+	/* T=1 Protocol Management for a TPDU reader */
+	if ((IFD_SUCCESS == ret)
+		&& (SCARD_PROTOCOL_T1 == ccid_descriptor->cardProtocol)
+		&& (CCID_CLASS_TPDU == (ccid_descriptor->dwFeatures & CCID_CLASS_EXCHANGE_MASK)))
+	{
+		/* timeout and cancel cases are faked by CCID_Receive() */
+		if (2 == *RxLength)
+		{
+			/* Decrement the sequence numbers since no TPDU was sent */
+			get_ccid_slot(reader_index)->t1.ns ^= 1;
+			get_ccid_slot(reader_index)->t1.nr ^= 1;
+		}
+		else
+		{
+			/* get only the T=1 data */
+			/* FIXME: manage T=1 error blocks */
+			memmove(RxBuffer, RxBuffer+3, *RxLength -4);
+			*RxLength -= 4;	/* remove NAD, PCB, LEN and CRC */
+		}
+	}
+
 	ccid_descriptor -> readTimeout = old_read_timeout;
 	return ret;
 } /* SecurePINVerify */
@@ -261,7 +326,7 @@ RESPONSECODE SecurePINVerify(unsigned int reader_index,
  *
  ****************************************************************************/
 RESPONSECODE SecurePINModify(unsigned int reader_index,
-	const unsigned char TxBuffer[], unsigned int TxLength,
+	unsigned char TxBuffer[], unsigned int TxLength,
 	unsigned char RxBuffer[], unsigned int *RxLength)
 {
 	unsigned char cmd[11+19+CMD_BUF_SIZE];
@@ -280,10 +345,23 @@ RESPONSECODE SecurePINModify(unsigned int reader_index,
 
 	/* 24 is the size of the PCSC PIN modify structure
 	 * The equivalent CCID structure is only 18 or 19-bytes long */
-	if ((TxLength > 19+CMD_BUF_SIZE) /* command too large? */
-		|| (TxLength < 18+4 /* 4 = APDU size */) /* command too short? */
-		|| (TxBuffer[20] + 24 != TxLength)) /* ulDataLength field coherency */
+	if (TxLength > 24+CMD_BUF_SIZE) /* command too large? */
 	{
+		DEBUG_INFO3("Command too long: %d > %d", TxLength, 24+CMD_BUF_SIZE);
+		*RxLength = 0;
+		return IFD_NOT_SUPPORTED;
+	}
+
+	if (TxLength < 24+4 /* 4 = APDU size */) /* command too short? */
+	{
+		DEBUG_INFO3("Command too short: %d < %d", TxLength, 24+4);
+		*RxLength = 0;
+		return IFD_NOT_SUPPORTED;
+	}
+
+	if (dw2i(TxBuffer, 20) + 24 != TxLength) /* ulDataLength field coherency */
+	{
+		DEBUG_INFO3("Wrong lengths: %d %d", dw2i(TxBuffer, 20) + 24, TxLength);
 		*RxLength = 0;
 		return IFD_NOT_SUPPORTED;
 	}
@@ -291,10 +369,92 @@ RESPONSECODE SecurePINModify(unsigned int reader_index,
 	/* Make sure in the beginning if bNumberMessage is valid or not */
 	if (TxBuffer[11] > 3)
 	{
+		DEBUG_INFO2("Wrong bNumberMessage: %d", TxBuffer[11]);
 		*RxLength = 0;
 		return IFD_NOT_SUPPORTED;
 	}
 
+	/* Make sure bEntryValidationCondition is valid
+	 * The Cherry XX44 reader crashes with a wrong value */
+	if ((0x00 == TxBuffer[10]) || (TxBuffer[10] > 0x07))
+	{
+		DEBUG_INFO2("Correct bEntryValidationCondition (was 0x%02X)",
+			TxBuffer[10]);
+		TxBuffer[10] = 0x02;
+	}
+
+#ifdef BOGUS_PINPAD_FIRMWARE
+	/* some firmwares are buggy so we try to "correct" the frame */
+	/*
+	 * SPR 532 and Cherry ST 2000C has no display but requires _all_
+	 * bMsgIndex fields with bNumberMessage set to 0.
+	 */
+	if ((SPR532 == ccid_descriptor->readerID)
+		|| (CHERRYST2000 == ccid_descriptor->readerID))
+	{
+		TxBuffer[11] = 0x03; /* set bNumberMessages to 3 so that
+								all bMsgIndex123 are filled */
+		TxBuffer[14] = TxBuffer[15] = TxBuffer[16] = 0;	/* bMsgIndex123 */
+	}
+
+	/* the bug is a bit different than for the Cherry ST 2000C
+	 * with bNumberMessages < 3 the command seems to be accepted
+	 * and the card sends 6B 80 */
+	if (CHERRYXX44 == ccid_descriptor->readerID)
+	{
+		TxBuffer[11] = 0x03; /* set bNumberMessages to 3 so that
+								all bMsgIndex123 are filled */
+	}
+
+	/* bug circumvention for the GemPC Pinpad */
+	if (GEMPCPINPAD == ccid_descriptor->readerID)
+	{
+		/* The reader does not support, and actively reject, "max size reached"
+		 * and "timeout occured" validation conditions */
+		if (0x02 != TxBuffer[10])
+		{
+			DEBUG_INFO2("Correct bEntryValidationCondition for GemPC Pinpad (was %d)",
+				TxBuffer[10]);
+			TxBuffer[10] = 0x02;	/* validation key pressed */
+		}
+
+		/* the reader does not support any other value than 3 for the number
+		 * of messages */
+		if (0x03 != TxBuffer[11])
+		{
+			DEBUG_INFO2("Correct bNumberMessages for GemPC Pinpad (was %d)",
+				TxBuffer[11]);
+			TxBuffer[11] = 0x03; /* 3 messages */
+		}
+	}
+#endif
+
+	/* T=1 Protocol Management for a TPDU reader */
+	if ((SCARD_PROTOCOL_T1 == ccid_descriptor->cardProtocol)
+		&& (CCID_CLASS_TPDU == (ccid_descriptor->dwFeatures & CCID_CLASS_EXCHANGE_MASK)))
+	{
+		ct_buf_t sbuf;
+		unsigned char sdata[T1_BUFFER_SIZE];
+
+		/* Initialize send buffer with the APDU */
+		ct_buf_set(&sbuf,
+			(void *)(TxBuffer + offsetof(PIN_MODIFY_STRUCTURE, abData)),
+			TxLength - offsetof(PIN_MODIFY_STRUCTURE, abData));
+
+		/* Create T=1 block */  
+		ret = t1_build(&((get_ccid_slot(reader_index))->t1),
+			sdata, 0, T1_I_BLOCK, &sbuf, NULL); 
+
+		/* Increment the sequence numbers  */
+		get_ccid_slot(reader_index)->t1.ns ^= 1;
+		get_ccid_slot(reader_index)->t1.nr ^= 1;
+
+		/* Copy the generated T=1 block prologue into the teoprologue
+		 * of the CCID command */
+		memcpy(TxBuffer + offsetof(PIN_MODIFY_STRUCTURE, bTeoPrologue),
+			sdata, 3);
+	}
+
 	/* Build a CCID block from a PC/SC V2.1.2 Part 10 block */
 
 	/* Do adjustments as needed - CCID spec is not exact with some
@@ -317,19 +477,6 @@ RESPONSECODE SecurePINModify(unsigned int reader_index,
 
 		if (16 == b) /* bMsgIndex3 */
 		{
-			/*
-			 * SPR 532 and Cherry ST 2000C has no display but requires _all_
-			 * bMsgIndex fields with bNumberMessage set to 0.
-			 */
-			if ((SPR532 == ccid_descriptor->readerID)
-				|| (CHERRYST2000 == ccid_descriptor->readerID))
-			{
-				cmd[21] = 0x00; /* set bNumberMessages to 0 */
-				cmd[a] = cmd[a-1] = cmd[a+1] = 0;	/* bMsgIndex123 */
-				a += 2;
-				continue;
-			}
-
 			/* in CCID the bMsgIndex3 is present only if bNumberMessage == 3 */
 			if (TxBuffer[11] < 3)
 				continue;
@@ -345,6 +492,14 @@ RESPONSECODE SecurePINModify(unsigned int reader_index,
 		a++;
  	}
 
+#ifdef BOGUS_PINPAD_FIRMWARE
+	if ((SPR532 == ccid_descriptor->readerID)
+		|| (CHERRYST2000 == ccid_descriptor->readerID))
+	{
+		cmd[21] = 0x00; /* set bNumberMessages to 0 */
+	}
+#endif
+
 	/* We know the size of the CCID message now */
 	i2dw(a - 10, cmd + 1);	/* command length (includes bPINOperation) */
 
@@ -359,6 +514,27 @@ RESPONSECODE SecurePINModify(unsigned int reader_index,
 
  	ret = CCID_Receive(reader_index, RxLength, RxBuffer);
 
+	/* T=1 Protocol Management for a TPDU reader */
+	if ((IFD_SUCCESS == ret)
+		&& (SCARD_PROTOCOL_T1 == ccid_descriptor->cardProtocol)
+		&& (CCID_CLASS_TPDU == (ccid_descriptor->dwFeatures & CCID_CLASS_EXCHANGE_MASK)))
+	{
+		/* timeout and cancel cases are faked by CCID_Receive() */
+		if (2 == *RxLength)
+		{
+			/* Decrement the sequence numbers since no TPDU was sent */
+			get_ccid_slot(reader_index)->t1.ns ^= 1;
+			get_ccid_slot(reader_index)->t1.nr ^= 1;
+		}
+		else
+		{
+			/* get only the T=1 data */
+			/* FIXME: manage T=1 error blocks */
+			memmove(RxBuffer, RxBuffer+3, *RxLength -4);
+			*RxLength -= 4;	/* remove NAD, PCB, LEN and CRC */
+		}
+	}
+
 	ccid_descriptor -> readTimeout = old_read_timeout;
 	return ret;
 } /* SecurePINModify */
