@@ -18,7 +18,7 @@
 */
 
 /*
- * $Id: commands.h 1630 2005-09-01 09:11:30Z rousseau $
+ * $Id: commands.h 1800 2006-01-16 13:45:50Z rousseau $
  */
 
 #define SIZE_GET_SLOT_STATUS 10
@@ -26,14 +26,14 @@
 #define ERROR_OFFSET 8
 
 RESPONSECODE CmdPowerOn(unsigned int reader_index, unsigned int * nlength,
-	unsigned char buffer[]);
+	unsigned char buffer[], int voltage);
 
 RESPONSECODE SecurePINVerify(unsigned int reader_index,
-	const unsigned char TxBuffer[], unsigned int TxLength,
+	unsigned char TxBuffer[], unsigned int TxLength,
 	unsigned char RxBuffer[], unsigned int *RxLength);
 
 RESPONSECODE SecurePINModify(unsigned int reader_index,
-	const unsigned char TxBuffer[], unsigned int TxLength,
+	unsigned char TxBuffer[], unsigned int TxLength,
 	unsigned char RxBuffer[], unsigned int *RxLength);
 
 RESPONSECODE CmdEscape(unsigned int reader_index,
