@@ -17,14 +17,15 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
 */
 
-/* $Id: ifdhandler.c 1752 2005-11-24 16:16:33Z rousseau $ */
+/* $Id: ifdhandler.c 2019 2006-04-20 18:57:14Z rousseau $ */
 
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
-#include <PCSC/pcsclite.h>
-#include <PCSC/ifdhandler.h>
-#include <PCSC/reader.h>
+#include "misc.h"
+#include <pcsclite.h>
+#include <ifdhandler.h>
+#include <reader.h>
 
 #include "ccid.h"
 #include "defs.h"
@@ -51,6 +52,7 @@ static pthread_mutex_t ifdh_context_mutex = PTHREAD_MUTEX_INITIALIZER;
 
 int LogLevel = DEBUG_LEVEL_CRITICAL | DEBUG_LEVEL_INFO;
 int DriverOptions = 0;
+int PowerOnVoltage = VOLTAGE_5V;
 static int DebugInitialized = FALSE;
 
 /* local functions */
@@ -63,7 +65,7 @@ static unsigned int T1_card_timeout(double f, double d, int TC1, int BWI,
 	int CWI, int clock_frequency);
 
 
-RESPONSECODE IFDHCreateChannelByName(DWORD Lun, LPTSTR lpcDevice)
+EXTERNAL RESPONSECODE IFDHCreateChannelByName(DWORD Lun, LPTSTR lpcDevice)
 {
 	RESPONSECODE return_value = IFD_SUCCESS;
 	int reader_index;
@@ -107,7 +109,7 @@ RESPONSECODE IFDHCreateChannelByName(DWORD Lun, LPTSTR lpcDevice)
 } /* IFDHCreateChannelByName */
 
 
-RESPONSECODE IFDHCreateChannel(DWORD Lun, DWORD Channel)
+EXTERNAL RESPONSECODE IFDHCreateChannel(DWORD Lun, DWORD Channel)
 {
 	/*
 	 * Lun - Logical Unit Number, use this for multiple card slots or
@@ -184,7 +186,7 @@ RESPONSECODE IFDHCreateChannel(DWORD Lun, DWORD Channel)
 } /* IFDHCreateChannel */
 
 
-RESPONSECODE IFDHCloseChannel(DWORD Lun)
+EXTERNAL RESPONSECODE IFDHCloseChannel(DWORD Lun)
 {
 	/*
 	 * This function should close the reader communication channel for the
@@ -225,7 +227,7 @@ RESPONSECODE IFDHCloseChannel(DWORD Lun)
 } /* IFDHCloseChannel */
 
 
-RESPONSECODE IFDHGetCapabilities(DWORD Lun, DWORD Tag,
+EXTERNAL RESPONSECODE IFDHGetCapabilities(DWORD Lun, DWORD Tag,
 	PDWORD Length, PUCHAR Value)
 {
 	/*
@@ -309,7 +311,7 @@ RESPONSECODE IFDHGetCapabilities(DWORD Lun, DWORD Tag,
 } /* IFDHGetCapabilities */
 
 
-RESPONSECODE IFDHSetCapabilities(DWORD Lun, DWORD Tag,
+EXTERNAL RESPONSECODE IFDHSetCapabilities(DWORD Lun, DWORD Tag,
 	/*@unused@*/ DWORD Length, /*@unused@*/ PUCHAR Value)
 {
 	/*
@@ -340,7 +342,7 @@ RESPONSECODE IFDHSetCapabilities(DWORD Lun, DWORD Tag,
 } /* IFDHSetCapabilities */
 
 
-RESPONSECODE IFDHSetProtocolParameters(DWORD Lun, DWORD Protocol,
+EXTERNAL RESPONSECODE IFDHSetProtocolParameters(DWORD Lun, DWORD Protocol,
 	UCHAR Flags, UCHAR PTS1, UCHAR PTS2, UCHAR PTS3)
 {
 	/*
@@ -466,6 +468,14 @@ RESPONSECODE IFDHSetProtocolParameters(DWORD Lun, DWORD Protocol,
 			ATR_GetParameter(&atr, ATR_PARAMETER_D, &d);
 			ATR_GetParameter(&atr, ATR_PARAMETER_F, &f);
 
+			/* may happen with non ISO cards */
+			if ((0 == f) || (0 == d))
+			{
+				/* values for TA1=11 */
+				f = 372;
+				d = 1;
+			}
+
 			/* Baudrate = f x D/F */
 			card_baudrate = (unsigned int) (1000 * ccid_desc->dwDefaultClock
 				* d / f);
@@ -490,8 +500,57 @@ RESPONSECODE IFDHSetProtocolParameters(DWORD Lun, DWORD Protocol,
 					DEBUG_COMM2("Set speed to %d bauds", card_baudrate);
 				}
 				else
+				{
 					DEBUG_COMM2("Reader does not support %d bauds",
 						card_baudrate);
+
+					/* TA2 present -> specific mode: the card is supporting
+					 * only the baud rate specified in TA1 but reader does not
+					 * support this value. Reject the card. */
+					if (atr.ib[1][ATR_INTERFACE_BYTE_TA].present)
+						return IFD_COMMUNICATION_ERROR;
+				}
+			}
+			else
+			{
+				/* the card is too fast for the reader */
+				if ((card_baudrate > ccid_desc->dwMaxDataRate +2)
+					/* but TA1 <= 97 */
+					&& (atr.ib[0][ATR_INTERFACE_BYTE_TA].value <= 0x97)
+					/* and the reader has a baud rate table */
+					&& ccid_desc->arrayOfSupportedDataRates)
+				{
+					unsigned char old_TA1;
+
+					old_TA1 = atr.ib[0][ATR_INTERFACE_BYTE_TA].value;
+					while (atr.ib[0][ATR_INTERFACE_BYTE_TA].value > 0x94)
+					{
+						/* use a lower TA1 */
+						atr.ib[0][ATR_INTERFACE_BYTE_TA].value--;
+
+						ATR_GetParameter(&atr, ATR_PARAMETER_D, &d);
+						ATR_GetParameter(&atr, ATR_PARAMETER_F, &f);
+
+						/* Baudrate = f x D/F */
+						card_baudrate = (unsigned int) (1000 *
+							ccid_desc->dwDefaultClock * d / f);
+
+						if (find_baud_rate(card_baudrate,
+							ccid_desc->arrayOfSupportedDataRates))
+						{
+							pps[1] |= 0x10; /* PTS1 presence */
+							pps[2] = atr.ib[0][ATR_INTERFACE_BYTE_TA].value;
+
+							DEBUG_COMM2("Set adapted speed to %d bauds",
+								card_baudrate);
+
+							break;
+						}
+					}
+
+					/* restore original TA1 value */
+					atr.ib[0][ATR_INTERFACE_BYTE_TA].value = old_TA1;
+				}
 			}
 		}
 	}
@@ -674,11 +733,14 @@ RESPONSECODE IFDHSetProtocolParameters(DWORD Lun, DWORD Protocol,
 		DEBUG_COMM3("T=1: IFSC=%d, IFSD=%d", t1->ifsc, t1->ifsd);
 	}
 
+	/* store used protocol for use by the secure commands (verify/change PIN) */
+	ccid_desc->cardProtocol = Protocol;
+
 	return IFD_SUCCESS;
 } /* IFDHSetProtocolParameters */
 
 
-RESPONSECODE IFDHPowerICC(DWORD Lun, DWORD Action,
+EXTERNAL RESPONSECODE IFDHPowerICC(DWORD Lun, DWORD Action,
 	PUCHAR Atr, PDWORD AtrLength)
 {
 	/*
@@ -755,7 +817,8 @@ RESPONSECODE IFDHPowerICC(DWORD Lun, DWORD Action,
 		case IFD_POWER_UP:
 		case IFD_RESET:
 			nlength = sizeof(pcbuffer);
-			if (CmdPowerOn(reader_index, &nlength, pcbuffer) != IFD_SUCCESS)
+			if (CmdPowerOn(reader_index, &nlength, pcbuffer, PowerOnVoltage)
+				!= IFD_SUCCESS)
 			{
 				DEBUG_CRITICAL("PowerUp failed");
 				return_value = IFD_ERROR_POWER_ACTION;
@@ -786,7 +849,7 @@ end:
 } /* IFDHPowerICC */
 
 
-RESPONSECODE IFDHTransmitToICC(DWORD Lun, SCARD_IO_HEADER SendPci,
+EXTERNAL RESPONSECODE IFDHTransmitToICC(DWORD Lun, SCARD_IO_HEADER SendPci,
 	PUCHAR TxBuffer, DWORD TxLength,
 	PUCHAR RxBuffer, PDWORD RxLength, /*@unused@*/ PSCARD_IO_HEADER RecvPci)
 {
@@ -842,8 +905,9 @@ RESPONSECODE IFDHTransmitToICC(DWORD Lun, SCARD_IO_HEADER SendPci,
 } /* IFDHTransmitToICC */
 
 
-RESPONSECODE IFDHControl(DWORD Lun, DWORD dwControlCode, PUCHAR TxBuffer,
-	DWORD TxLength, PUCHAR RxBuffer, DWORD RxLength, PDWORD pdwBytesReturned)
+EXTERNAL RESPONSECODE IFDHControl(DWORD Lun, DWORD dwControlCode,
+	PUCHAR TxBuffer, DWORD TxLength, PUCHAR RxBuffer, DWORD RxLength,
+	PDWORD pdwBytesReturned) 
 {
 	/*
 	 * This function performs a data exchange with the reader (not the
@@ -953,7 +1017,7 @@ RESPONSECODE IFDHControl(DWORD Lun, DWORD dwControlCode, PUCHAR TxBuffer,
 } /* IFDHControl */
 
 
-RESPONSECODE IFDHICCPresence(DWORD Lun)
+EXTERNAL RESPONSECODE IFDHICCPresence(DWORD Lun)
 {
 	/*
 	 * This function returns the status of the card inserted in the
@@ -1022,6 +1086,7 @@ RESPONSECODE IFDHICCPresence(DWORD Lun)
 
 	/* SCR331-DI contactless reader */
 	if (((SCR331DI == ccid_descriptor->readerID)
+		|| (SDI010 == ccid_descriptor->readerID)
 		|| (SCR331DINTTCOM == ccid_descriptor->readerID))
 		&& (ccid_descriptor->bCurrentSlotIndex > 0))
 	{
@@ -1100,6 +1165,26 @@ void init_driver(void)
 		DEBUG_INFO2("DriverOptions: 0x%.4X", DriverOptions);
 	}
 
+	/* get the voltage parameter */
+	switch ((DriverOptions >> 4) & 0x03)
+	{
+		case 0:
+			PowerOnVoltage = VOLTAGE_5V;
+			break;
+
+		case 1:
+			PowerOnVoltage = VOLTAGE_3V;
+			break;
+
+		case 2:
+			PowerOnVoltage = VOLTAGE_1_8V;
+			break;
+
+		case 3:
+			PowerOnVoltage = VOLTAGE_AUTO;
+			break;
+	}
+
 	/* initialise the Lun to reader_index mapping */
 	InitReaderIndex();
 
@@ -1133,6 +1218,10 @@ void extra_egt(ATR_t *atr, _ccid_descriptor *ccid_desc, DWORD Protocol)
 	ATR_GetParameter(atr, ATR_PARAMETER_D, &d);
 	ATR_GetParameter(atr, ATR_PARAMETER_F, &f);
 
+	/* may happen with non ISO cards */
+	if ((0 == f) || (0 == d))
+		return;
+
 	/* Baudrate = f x D/F */
 	card_baudrate = (unsigned int) (1000 * ccid_desc->dwDefaultClock * d / f);
 
@@ -1185,7 +1274,7 @@ static char find_baud_rate(unsigned int baudrate, unsigned int *list)
 
 	DEBUG_COMM2("Card baud rate: %d", baudrate);
 
-	/* Does the reader support the annonced smart card data speed? */
+	/* Does the reader support the announced smart card data speed? */
 	for (i=0;; i++)
 	{
 		/* end of array marker */
@@ -1233,6 +1322,10 @@ static unsigned int T0_card_timeout(double f, double d, int TC1, int TC2,
 	/* clock_frequency is in kHz so the times are in milliseconds and not
 	 * in seconds */
 
+	/* may happen with non ISO cards */
+	if ((0 == f) || (0 == d) || (0 == clock_frequency))
+		return 60;	/* 60 seconds */
+
 	/* EGT */
 	/* see ch. 6.5.3 Extra Guard Time, page 12 of ISO 7816-3 */
 	EGT = 12 * f / d / clock_frequency + (f / d) * TC1 / clock_frequency;
@@ -1279,6 +1372,10 @@ static unsigned int T1_card_timeout(double f, double d, int TC1,
 	/* clock_frequency is in kHz so the times are in milliseconds and not
 	 * in seconds */
 
+	/* may happen with non ISO cards */
+	if ((0 == f) || (0 == d) || (0 == clock_frequency))
+		return 60;	/* 60 seconds */
+
 	/* see ch. 6.5.2 Transmission factors F and D, page 12 of ISO 7816-3 */
 	etu = f / d / clock_frequency;
 
