@@ -7,25 +7,21 @@
  * Copyright (C) 2004 Ludovic Rousseau <ludovic.rousseau@free.fr>
  */
 
-#include <PCSC/pcsclite.h>
-#include <PCSC/ifdhandler.h>
+#include <pcsclite.h>
+#include <ifdhandler.h>
 #include "commands.h"
 #include "buffer.h"
 #include "debug.h"
 #include "proto-t1.h"
 #include "checksum.h"
 
+#include "ccid.h"
+
 #include <sys/poll.h>
 #include <unistd.h>
 #include <stdlib.h>
 #include <string.h>
 
-/* T=1 protocol constants */
-#define T1_I_BLOCK		0x00
-#define T1_R_BLOCK		0x80
-#define T1_S_BLOCK		0xC0
-#define T1_MORE_BLOCKS		0x20
-
 /* I block */
 #define T1_I_SEQ_SHIFT		6
 
@@ -65,9 +61,6 @@ enum {
 static void		t1_set_checksum(t1_state_t *, int);
 static unsigned	int	t1_block_type(unsigned char);
 static unsigned int	t1_seq(unsigned char);
-static unsigned	int	t1_build(t1_state_t *, unsigned char *,
-				unsigned char, unsigned char,
-				ct_buf_t *, size_t *);
 static unsigned int	t1_rebuild(t1_state_t *t1, unsigned char *block);
 static unsigned int	t1_compute_checksum(t1_state_t *,
 				unsigned char *, size_t);
@@ -556,7 +549,7 @@ t1_seq(unsigned char pcb)
 	}
 }
 
-static unsigned int
+unsigned int
 t1_build(t1_state_t *t1, unsigned char *block,
 		unsigned char dad, unsigned char pcb,
 		ct_buf_t *bp, size_t *lenp)
@@ -653,8 +646,22 @@ static int
 t1_xcv(t1_state_t *t1, unsigned char *block, size_t slen, size_t rmax)
 {
 	int		n, m;
+	_ccid_descriptor *ccid_desc ;
+	int oldReadTimeout;
+	unsigned int rmax_int;
 
 	DEBUG_XXD("sending: ", block, slen);
+	
+	ccid_desc = get_ccid_descriptor(t1->lun);	
+	oldReadTimeout = ccid_desc->readTimeout;
+
+	if (t1->wtx > 1)
+	{	
+		/* set the new temporary timeout at WTX card request */
+		ccid_desc->readTimeout *=  t1->wtx;
+		DEBUG_INFO2("New timeout at WTX request: %d sec",
+			ccid_desc->readTimeout);
+	}
 
 	if (isCharLevel(t1->lun))
 	{
@@ -664,7 +671,13 @@ t1_xcv(t1_state_t *t1, unsigned char *block, size_t slen, size_t rmax)
 		if (n != IFD_SUCCESS)
 			return n;
 
-		n = CCID_Receive(t1 -> lun, &rmax, block);
+		/* the second argument of CCID_Receive() is (unsigned int *)
+		 * so we can't use &rmax since &rmax is a (size_t *) and may not
+		 * be the same on 64-bits architectures for example (iMac G5) */
+		rmax_int = rmax;
+		n = CCID_Receive(t1 -> lun, &rmax_int, block);
+		rmax = rmax_int;
+
 		if (n == IFD_PARITY_ERROR)
 			return -2;
 		if (n != IFD_SUCCESS)
@@ -676,7 +689,9 @@ t1_xcv(t1_state_t *t1, unsigned char *block, size_t slen, size_t rmax)
 		if (n != IFD_SUCCESS)
 			return n;
 
-		n = CCID_Receive(t1 -> lun, &rmax, &block[3]);
+		rmax_int = rmax;
+		n = CCID_Receive(t1 -> lun, &rmax_int, &block[3]);
+		rmax = rmax_int;
 		if (n == IFD_PARITY_ERROR)
 			return -2;
 		if (n != IFD_SUCCESS)
@@ -692,7 +707,9 @@ t1_xcv(t1_state_t *t1, unsigned char *block, size_t slen, size_t rmax)
 			return n;
 
 		/* Get the response en bloc */
-		n = CCID_Receive(t1 -> lun, &rmax, block);
+		rmax_int = rmax;
+		n = CCID_Receive(t1 -> lun, &rmax_int, block);
+		rmax = rmax_int;
 		if (n == IFD_PARITY_ERROR)
 			return -2;
 		if (n != IFD_SUCCESS)
@@ -711,6 +728,9 @@ t1_xcv(t1_state_t *t1, unsigned char *block, size_t slen, size_t rmax)
 	if (n >= 0)
 		DEBUG_XXD("received: ", block, n);
 
+	/* Restore initial timeout */
+	ccid_desc->readTimeout = oldReadTimeout;
+
 	return n;
 }
 
