@@ -18,10 +18,10 @@
 */
 
 /*
- * $Id: utils.c,v 1.12 2005/03/14 18:56:52 rousseau Exp $
+ * $Id: utils.c 1771 2005-11-29 12:21:00Z rousseau $
  */
 
-#include <PCSC/pcsclite.h>
+#include <pcsclite.h>
 
 #include "ccid.h"
 #include "defs.h"
